##
##---------------------------------------------------------------------------##
##
## PySol -- a Python Solitaire game
##
## Copyright (C) 2000 by T. Kirk
## Copyright (C) 2000 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1999 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1998 Markus Franz Xaver Johannes Oberhumer
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING.
## If not, write to the Free Software Foundation, Inc.,
## 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##
## Markus F.X.J. Oberhumer
## <markus.oberhumer@jk.uni-linz.ac.at>
## http://wildsau.idv.uni-linz.ac.at/mfx/pysol.html
##
## T. Kirk
## <grania@subdimension.com>
## http://www.inetarena.com/~grania
##
##---------------------------------------------------------------------------##

import sys

# PySol imports
if sys.modules.has_key("pysoltk"):
    from hanafuda import *



# /***********************************************************************
#  * Flower Foundation Stacks
#  ***********************************************************************/

# /***********************************************************************
#  * Flower Row Stacks
#  ***********************************************************************/

class JapaneseGarden_RowStack(Flower_OpenStack):

    def acceptsCards(self, from_stack, cards):
        if (not self.basicAcceptsCards(from_stack, cards)
		or not from_stack in self.game.s.rows):
            return 0
        stackcards = self.cards
	if not len(stackcards):
	    return 1
        return stackcards[-1].rank + 1 == cards[0].rank


class HanafudaRK_RowStack(Flower_OpenStack):

    def acceptsCards(self, from_stack, cards):
        if (not self.basicAcceptsCards(from_stack, cards)
		or not isRankSequence(cards, dir=1)):
            return 0
        stackcards = self.cards
	if not len(stackcards):
	    return 1
        return stackcards[-1].rank + 1 == cards[0].rank


# /***********************************************************************
# * Japanese Garden
# ************************************************************************/

class JapaneseGarden(AbstractFlowerGame):
    RowStack_Class = FlowerClock_RowStack
    WIDTH = 9
    HEIGHT = 6
    XROWS = 3
    YROWS = 2
    MAX_CARDS = 6
    MAX_MOVE = 1
    XRESERVES = 6
    YRESERVES = 2
    MAX_RESERVE = 0
    INITIAL_DEAL = 6
    DEAL_RESERVES = 1

    #
    # Game layout
    #

    def createGame(self):
        l, s = Layout(self), self.s
        font = getFont("canvas_card", cardw=l.CW)

        # Set window size
        self.setSize(l.XM + l.XS * self.WIDTH, l.YM * 3 + l.YS * self.HEIGHT)

        # Create foundations
	x = self.width / 2 + l.XM / 2 - l.XS * 3
        y = l.YM
	for j in range(2):
	    for i in range(6):
		s.foundations.append(Hanafuda_SS_FoundationStack(x, y, self, i + (j * 6),
                                    max_cards=4, max_accept=1, base_rank=3))
		x = x + l.XS
	    x = self.width / 2 + l.XM / 2 - l.XS * 3
	    y = y + l.YS

        # Create flower beds
	x = l.XM
	y = l.YM * 2 + l.YS * 2
	for j in range(self.YROWS):
	    for i in range(self.XROWS):
		row = self.RowStack_Class(x, y, self, yoffset=0, max_accept=self.MAX_MOVE,
				max_move=self.MAX_MOVE, max_cards=self.MAX_CARDS, base_rank=0)
		row.CARD_XOFFSET = l.CW / 2
		s.rows.append(row)
		x = x + (self.width - l.XM) / self.XROWS
	    x = l.XM
	    y = y + l.YS
	self.setRegion(s.rows, (l.XM, l.YS * 2, 999999, y))

	# Create pool
	x = self.width / 2 + l.XM / 2 - (l.XS * self.XRESERVES) / 2
	for j in range(self.YRESERVES):
	    for i in range(self.XRESERVES):
		s.reserves.append(ReserveStack(x, y, self, max_accept=self.MAX_RESERVE))
		x = x + l.XS
	    x = self.width / 2 + l.XM / 2 - l.XS * (self.XRESERVES / 2)
	    y = y + l.YS
	if s.reserves:
	    self.setRegion(s.reserves, (l.XM, l.YS * (2 + self.YROWS), 999999, 999999))

        # Create talon
	s.talon = InitialDealTalonStack(l.XM, l.YM, self)

        # Define stack-groups
        l.defaultStackGroups()

    #
    # Game over rides
    #

    def startGame(self):
        assert len(self.s.talon.cards) == 48
	for i in range(self.INITIAL_DEAL):
	    self.s.talon.dealRow()
	if self.DEAL_RESERVES:
	    self.s.talon.dealRow(rows=self.s.reserves)
        self.startDealSample()
        self.s.talon.dealCards()



# /***********************************************************************
# * Japanese Garden II
# ************************************************************************/

class JapaneseGardenII(JapaneseGarden):
    RowStack_Class = JapaneseGarden_RowStack



# /***********************************************************************
# * Japanese Garden III
# ************************************************************************/

class JapaneseGardenIII(JapaneseGardenII):
    XROWS = 2
    YROWS = 4
    MAX_CARDS = 7
    XRESERVES = 0
    YRESERVES = 0
    DEAL_RESERVES = 0



# /***********************************************************************
# * Six Sages
# ************************************************************************/

class SixSages(JapaneseGarden):
    XROWS = 2
    YROWS = 3
    MAX_CARDS = 9
    XRESERVES = 1
    YRESERVES = 1
    MAX_RESERVE = 1
    INITIAL_DEAL = 8
    DEAL_RESERVES = 0



# /***********************************************************************
# * Six Tengus
# ************************************************************************/

class SixTengus(SixSages):
    RowStack_Class = HanafudaRK_RowStack
    HEIGHT = 5
    MAX_MOVE = 2
    XRESERVES = 0
    YRESERVES = 0



registerGame(GameInfo(12373, JapaneseGarden, "Japanese Garden",
                      GI.GT_HANAFUDA | GI.GT_OPEN, 1, 0,
                      suits=range(12), ranks=range(4)))

registerGame(GameInfo(12374, JapaneseGardenII, "Japanese Garden II",
                      GI.GT_HANAFUDA | GI.GT_OPEN, 1, 0,
                      suits=range(12), ranks=range(4)))

registerGame(GameInfo(12375, SixSages, "Six Sages",
                      GI.GT_HANAFUDA | GI.GT_OPEN, 1, 0,
                      suits=range(12), ranks=range(4)))

registerGame(GameInfo(12376, SixTengus, "Six Tengus",
                      GI.GT_HANAFUDA | GI.GT_OPEN, 1, 0,
                      suits=range(12), ranks=range(4)))

registerGame(GameInfo(12377, JapaneseGardenIII, "Japanese Garden III",
                      GI.GT_HANAFUDA | GI.GT_OPEN, 1, 0,
                      suits=range(12), ranks=range(4)))

